﻿/*
Main document class
*/

package {
	import flash.events.*;
	import flash.display.*;

	import caurina.transitions.Tweener;

	import org.papervision3d.cameras.*;
	import org.papervision3d.events.*;
	import org.papervision3d.materials.*;
	import org.papervision3d.objects.*;
	import org.papervision3d.scenes.*;

	public class Main extends Sprite {

		// Scene stuff
		private var container:Sprite;
		private var scene:Scene3D;
		private var camera:Camera3D;
		private var coneYellow:Ase;
		private var coneBlack:Ase;
		private var rootNode:DisplayObject3D;

		public var basePanel:MovieClip;

		private var viewXY:Viewport;
		private var viewXZ:Viewport;
		private var viewZY:Viewport;

		private var coneList:Array;

		// Curve objects
		public var p1:Object;
		public var p2:Object;

		// Generic stuff
		var selectedPoint:Number;
		var controlPoints:Array;
		var curvePoints:Array;
		var targetTravel:Boolean;

		public function Main() {
			stage.quality = "HIGH";
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			stage.addEventListener(Event.RESIZE, onResize);

			coneList = new Array();

			// 3d scene stuff -------------------------------------------------------------------------------------

			// Creates scene
			container = new Sprite();
			addChild(container);
			scene = new Scene3D(container);
			rootNode = new DisplayObject3D("rootNode");
			scene.addChild(rootNode);
			
			// Creates cameras
			camera = new Camera3D();
			camera.x = 0;
			camera.z = 2000;
			camera.y = 1000;
			camera.zoom = 4;
			camera.focus = 100;
			targetTravel = true;

			p1 = {x:-1900, y:1500, z:-1800};
			p2 = {x:-325, y:950, z:-1350};

			// Viewports
			viewXY = new Viewport("xy");
			addChild(viewXY);
			viewXZ = new Viewport("xz");
			addChild(viewXZ);
			viewZY = new Viewport("zy");
			addChild(viewZY);

			// Creates objects
			addCone (0, 0, 0);

			addCone (-800, 0, 0);
			addCone (800, 0, 0);

			addCone (0, 0, -800);

			addEventListener(Event.ENTER_FRAME, tick); // I like UT

			// 2d interface stuff ---------------------------------------------------------------------------------

			selectedPoint = NaN;
			controlPoints = [];
			curvePoints = [];
			
			// Sample control points
			addControlPoint(-575, 450, -375);
			addControlPoint(412, 275, -337);
			addControlPoint(344, 113, 370);
			addControlPoint(-466, 56, 398);
			addControlPoint(-336, 304, -1177);

			// Setup buttons
			basePanel.bAdd.addEventListener(MouseEvent.CLICK, function() {
				addControlPoint();
			});
			basePanel.bRemove.addEventListener(MouseEvent.CLICK, function() {
				removeCurrentPoint();
			});
			basePanel.bStart.addEventListener(MouseEvent.CLICK, function() {
				startTravel();
			});
			basePanel.bStop.addEventListener(MouseEvent.CLICK, function() {
				stopTravel();
			});
			basePanel.bTarget.addEventListener(MouseEvent.CLICK, function() {
				targetTravel = !targetTravel;
				updateCode();
				stopTravel();
			});
			basePanel.bCode.addEventListener(MouseEvent.CLICK, function() {
				basePanel.codeCaption.visible = !basePanel.codeCaption.visible;
			});
			basePanel.bFullscreen.addEventListener(MouseEvent.CLICK, function() {
				if (stage.displayState == "fullScreen") {
					stage.displayState = "normal";
				} else {
					stage.displayState = "fullScreen";
				}
			});

			setChildIndex(basePanel, numChildren - 1);
			setChildIndex(viewXY, numChildren - 1);
			setChildIndex(viewXZ, numChildren - 1);
			setChildIndex(viewZY, numChildren - 1);
//			basePanel.visible = false;
			basePanel.codeCaption.visible = false;
	
			onResize();
			redrawViewports();
//			startTravel();
	
		}

		private function tick(event:Event):void {
			// A tick of the engine
			scene.renderCamera(camera);
		}

		private function updateCode():void {
			// Creates the code
			var i:uint;
			var animTime:Number = Number(basePanel.timeCaption.text);

			var cs:String = "";
			cs += "// Reset the position of the MovieClip\n";
			cs += "camera.x = " + p1.x + ";\n";
			cs += "camera.y = " + p1.y + ";\n";
			cs += "camera.z = " + p1.z + ";\n";
			cs += "\n";
		
			// Creates list of control points
			var bezierList:Array = [];
			for (i = 0; i < controlPoints.length; i++) {
				bezierList.push("{x:"+Math.round(controlPoints[i].x)+", y:"+Math.round(controlPoints[i].y)+", z:"+Math.round(controlPoints[i].z)+"}");
			}
			var bezierParameter:String;
			if (bezierList.length == 0) {
				// No bezier point
				bezierParameter = "";
			} else if (bezierList.length == 1) {
				// Single control point, no array is needed
				bezierParameter = ", _bezier:" + bezierList[0];
			} else {
				// Array of control points
				bezierParameter = ", _bezier:[" + bezierList.join(", ") + "]";
			}
		
			if (targetTravel) {
				cs += "// Target moves on the path first\n";
				cs += "camera.target.x = " + p1.x + ";\n";
				cs += "camera.target.y = " + p1.y + ";\n";
				cs += "camera.target.z = " + p1.z + ";\n";
				cs += "Tweener.addTween(camera.target, {x:" + p2.x + ", y:" + p2.y + bezierParameter + ", time:"+animTime+", transition:\"linear\"});\n";
				cs += "\n";
			} else {
				cs += "// Target is fixed on the center\n";
				cs += "camera.target.x = 0;\n";
				cs += "camera.target.y = 100;\n";
				cs += "camera.target.z = 0;\n";
				cs += "\n";
			}
			cs += "// Animate camera position\n";
			cs += "Tweener.addTween(camera, {x:" + p2.x + ", y:" + p2.y + ", z:" + p2.z + bezierParameter + ", time:"+animTime+", "+(targetTravel ? "delay:"+(animTime/20)+", " : "")+"transition:\"linear\"});\n";
		
			basePanel.codeCaption.text = cs;
		}

		private function addCone(p_x:Number, p_y:Number, p_z:Number):void {
			// Adds a cone to the entire scene
			var coneIndex:uint = coneList.length;

			var newConeMaterial:BitmapAssetMaterial = new BitmapAssetMaterial(coneIndex % 2 == 1 ? "ConeDiffuseYellow" : "ConeDiffuseBlack");
			var newCone:Ase = new Ase(newConeMaterial, "cone.ase", 0.1);
			rootNode.addChild(newCone);
			coneList[coneIndex] = newCone;
			newCone.x = p_x;
			newCone.y = p_y;
			newCone.z = p_z;

			viewXY.addCone(coneIndex);
			viewXZ.addCone(coneIndex);
			viewZY.addCone(coneIndex);
			
		}

		public function getCone(p:Number):Ase {
			return coneList[p];
		}
		public function getControlPoint(p:Number):Object {
			return controlPoints[p];
		}

		public function redrawViewports():void {
			updateCode();
			stopTravel();
			viewXY.redraw();
			viewXZ.redraw();
			viewZY.redraw();
		}


		public function addControlPoint(p_x:Number = NaN, p_y:Number = NaN, p_z:Number = NaN, p:Number = NaN):void {
			// Adds a control point
			var i:Number;
			
			if (isNaN(p)) {
				// Finds the good index for a new point based on the selected point
				if (isNaN(selectedPoint)) {
					// Last
					p = controlPoints.length;
				} else {
					// Last by default?
					p = controlPoints.length;
					for (i = 0; i < controlPoints.length; i++) {
						if (controlPoints[i] == selectedPoint) {
							p = i + 1;
							break;
						}
					}
				}
					
			}
		
			if (isNaN(p_x) || isNaN(p_y)) {
				// Find the better position
				var lPoint:Object = (p == 0) ? p1 : controlPoints[p-1];
				var rPoint:Object = (p == controlPoints.length) ? p2 : controlPoints[p];
				p_x = (lPoint.x + rPoint.x)/2;
				p_y = (lPoint.y + rPoint.y)/2;
				p_z = (lPoint.z + rPoint.z)/2;
			}
			var cp:Object = {x:p_x, y:p_y, z:p_z};
			controlPoints.splice(p, 0, cp);

			viewXY.addControlPoint(p);
			viewXZ.addControlPoint(p);
			viewZY.addControlPoint(p);

			stopTravel();
			updateCode();
			
		}

		public function removeControlPoint(p:Number):void {
			// Removes the last control point
			var cp:Object;
			if (p == selectedPoint) setSelectedPoint();
			if (controlPoints.length > 0) {
				cp = controlPoints[p];
				controlPoints.splice(p, 1);
				viewXY.removeControlPoint(p);
				viewXZ.removeControlPoint(p);
				viewZY.removeControlPoint(p);
				stopTravel();
				updateCode();
			}

		}

		public function setSelectedPoint(p:Number = NaN):void {
			// Selects a point
			if (p == selectedPoint) return;
			
			// Deselect current
			if (!isNaN(selectedPoint)) setPointAlpha(selectedPoint, 1);
		
			// Select new
			selectedPoint = p;
			if (!isNaN(selectedPoint)) setPointAlpha(selectedPoint, 2);
		}

		public function setPointAlpha(p:Number, p_alpha:Number):void {
			if (isNaN(p)) return;
			viewXY.setPointAlpha(p, p_alpha);
			viewXZ.setPointAlpha(p, p_alpha);
			viewZY.setPointAlpha(p, p_alpha);
		}

		public function removeCurrentPoint():void {
			// Removes the point currently selected only if it's a bezier point
			var i:uint;

			if (!isNaN(selectedPoint)) {
				removeControlPoint(selectedPoint);
				setSelectedPoint();
			}
		}

		public function stopTravel(): void {
			Tweener.removeTweens(camera);
			Tweener.removeTweens(camera.target);
			camera.x = p1.x;
			camera.y = p1.y;
			camera.z = p1.z;
			camera.target.x = 0;
			camera.target.y = 100;
			camera.target.z = 0;
			viewXY.stopTravel();
			viewXZ.stopTravel();
			viewZY.stopTravel();
		}

		public function startTravel():void {
			// Plays the ball using the bezier values
			stopTravel();

			var i:uint;
			var animTime:Number = Number(basePanel.timeCaption.text);
			var animDelay:Number = targetTravel ? animTime/20 : 0;
		
			// Creates list of control points
			var bezierList:Array = [];
			for (i = 0; i < controlPoints.length; i++) {
				bezierList.push({x:controlPoints[i].x, y:controlPoints[i].y, z:controlPoints[i].z});
			}
			Tweener.addTween(camera, {x:p2.x, y:p2.y, z:p2.z, _bezier:bezierList, delay:animDelay, time:animTime, transition:"linear"});
			if (targetTravel) {
				// Algo tweens the target
				camera.target.x = p1.x;
				camera.target.y = p1.y;
				camera.target.z = p1.z;
				Tweener.addTween(camera.target, {x:p2.x, y:p2.y, z:p2.z, _bezier:bezierList, time:animTime, transition:"linear"});
			}
		
			viewXY.startTravel(bezierList, animTime, animDelay);
			viewXZ.startTravel(bezierList, animTime, animDelay);
			viewZY.startTravel(bezierList, animTime, animDelay);
		}

		public function onResize(e:Event = null):void {
			container.x = Math.round(stage.stageWidth/2);
			container.y = Math.round(stage.stageHeight/2);

			var viewPortWidth:Number = Math.floor((stage.stageWidth-2) / 3);
			var viewPortHeight:Number = Math.floor(stage.stageHeight / 4);
			viewXY.setSize(viewPortWidth, viewPortHeight);
			viewXY.x = 0;
			viewXY.y = stage.stageHeight - viewPortHeight;
			viewXZ.setSize(viewPortWidth, viewPortHeight);
			viewXZ.x = viewPortWidth+1;
			viewXZ.y = stage.stageHeight - viewPortHeight;
			viewZY.setSize(stage.stageWidth - ((viewPortWidth*2)+2), viewPortHeight);
			viewZY.x = viewPortWidth*2+2;
			viewZY.y = stage.stageHeight - viewPortHeight;

			basePanel.y = stage.stageHeight - 160;
			basePanel.y = 0;
			basePanel.codeCaption.width = stage.stageWidth;
			basePanel.codeCaption.height = stage.stageHeight - basePanel.codeCaption.y;
		}

	}
}
