/*
Viewport that allows different views
*/

package {
	import flash.events.*;
	import flash.display.*;
	import flash.text.TextField;

	import org.papervision3d.cameras.*;
	import org.papervision3d.events.*;
	import org.papervision3d.materials.*;
	import org.papervision3d.objects.*;
	import org.papervision3d.scenes.*;

	import caurina.transitions.Tweener;

	public class Viewport extends Sprite {

		private var ball:TravelBall;
		// Scene stuff
		private var coneList:Array;
		public var controlPoints:Array;
		private var curvePoints:Array;
		private var sizeW:Number;
		private var sizeH:Number;
		private var viewType:String; // xy, xz, zy
		private var d1:String, d2:String;

		private static var CONE_SCALE:Number = 0.04;

		private var p1:DraggablePoint;
		private var p2:DraggablePoint;

		private var caption:TextField;
//		private var pLine:MovieClip;

		public function Viewport(p_type:String) {
			coneList = new Array();
			controlPoints = new Array();
			curvePoints = [];
			viewType = p_type.toLowerCase();
			d1 = viewType.substr(0, 1);
			d2 = viewType.substr(1, 1);
			setSize(100, 100);
			caption = new TextField();
			addChild(caption);
			caption.text = viewType.toUpperCase();
			caption.selectable = false;
			p1 = new DraggablePoint();
			addChild(p1);
			p2 = new DraggablePoint();
			addChild(p2);
			
			ball = new TravelBall();
			addChild(ball);
			ball.scaleX = ball.scaleY = 0.8;
			ball.alpha = 0.7;

//			pLine = new MovieClip();
//			addChild(pLine);

		}

		public function setSize(p_w:Number, p_h:Number):void {
			sizeW = p_w;
			sizeH = p_h;
			redraw();
		}

		public function addCone(p_index:Number):void {
			var coneIcon:Sprite = new DraggableCone(viewType == "xz", p_index);
			coneList[p_index] = coneIcon;
			addChild(coneIcon);
			redraw();
		}

		public function addControlPoint(p_index:Number):void {
			var cPoint:Sprite = new DraggableControlPoint();
			controlPoints[p_index] = cPoint;
			addChild(cPoint);
			redraw();
		}

		public function removeControlPoint(p_index:Number):void {
			var cm:MovieClip = controlPoints[p_index];
			controlPoints.splice(p_index, 1);
			removeChild(cm);
			redraw();
		}


		public function redraw():void {
			// Draws background
			graphics.clear();
			graphics.lineStyle();
			graphics.beginFill(0xf1f1f1, 1);
			graphics.moveTo(0, 0);
			graphics.lineTo(sizeW, 0);
			graphics.lineTo(sizeW, sizeH);
			graphics.lineTo(0, sizeH);
			graphics.endFill();

			graphics.lineStyle(1, 0x000000, 0.1);
			graphics.moveTo(sizeW/2, 0);
			graphics.lineTo(sizeW/2, sizeH);
			graphics.moveTo(0, sizeH/2);
			graphics.lineTo(sizeW, sizeH/2);

			var i:uint;
			var offsetX:Number = sizeW/2;
			var offsetY:Number = sizeH/2;

			// Draws icons
			for (i = 0; i < coneList.length; i++) {
				var mainCone:Ase = Main(parent).getCone(i);
				var localCone:Sprite = coneList[i];
				localCone.x = offsetX + mainCone[d1] * CONE_SCALE;
				localCone.y = offsetY + mainCone[d2] * CONE_SCALE * -1;
			}

			// Draw points
			if (Boolean(parent)) {
				p1.x = offsetX + Main(parent).p1[d1] * CONE_SCALE;
				p1.y = offsetY + Main(parent).p1[d2] * CONE_SCALE * -1;
				p2.x = offsetX + Main(parent).p2[d1] * CONE_SCALE;
				p2.y = offsetY + Main(parent).p2[d2] * CONE_SCALE * -1;
			}

			for (i = 0; i < controlPoints.length; i++) {
				var mainPoint:Object = Main(parent).getControlPoint(i);
				var localPoint:Sprite = controlPoints[i];
				localPoint.x = offsetX + mainPoint[d1] * CONE_SCALE;
				localPoint.y = offsetY + mainPoint[d2] * CONE_SCALE * -1;
			}

			redrawPath();
		}

		public function redrawPath():void {

			if (!Boolean(p1)) return;

			var i:uint;
		
			for (i = 0; i < curvePoints.length; i++) {
				removeChild(curvePoints[i]);
				curvePoints[i] = null;
			}
			curvePoints = [];
		
			// Draws path curve and curve points
			var g:Graphics = graphics;
			g.lineStyle(2, 0x000000, 0.8);
			var startX:Number = p1.x;
			var startY:Number = p1.y;
			var endX:Number, endY:Number;
			var thisPoint:MovieClip;
			var nextPoint:MovieClip;
			g.moveTo(startX, startY);
			for (i = 0; i < controlPoints.length-1; i++) {
				thisPoint = controlPoints[i];
				nextPoint = controlPoints[i+1];
				endX = (thisPoint.x + nextPoint.x) / 2;
				endY = (thisPoint.y + nextPoint.y) / 2;
				g.curveTo(thisPoint.x, thisPoint.y, endX, endY);
				// Draw curve points for reference
				drawCurvePoints(startX, startY, thisPoint.x, thisPoint.y, endX, endY);
				startX = endX;
				startY = endY;
			}
			if (controlPoints.length > 0) {
				// Normal curve
				var lastPoint:MovieClip = controlPoints[controlPoints.length - 1];
				g.curveTo(lastPoint.x, lastPoint.y, p2.x, p2.y);
				drawCurvePoints(startX, startY, lastPoint.x, lastPoint.y, p2.x, p2.y);
			} else {
				// Just a line!
				g.lineTo(p2.x, p2.y);
				drawLinePoints(startX, startY, p2.x, p2.y);
			}
			
			// Draw bezier control lines
			g.lineStyle(1, 0x000000, 0.2);
			g.moveTo(p1.x, p1.y);
			for (i = 0; i < controlPoints.length; i++) {
				thisPoint = controlPoints[i];
				g.lineTo(thisPoint.x, thisPoint.y);
			}
			g.lineTo(p2.x, p2.y);

			// Move all draggable buttons to the top
			setChildIndex(p1, numChildren - 1);
			setChildIndex(p2, numChildren - 1);
			for (i = 0; i < controlPoints.length; i++) setChildIndex(controlPoints[i], numChildren - 1);
			
			stopTravel();
			
		}

		public function drawCurvePoints(px1:Number, py1:Number, cx:Number, cy:Number, px2:Number, py2:Number):void {
			// Draws points on curves
			var i:Number;
			for (i = 0; i < 1; i+= 1/5) {
				var pt = pennerPointOnCurve(px1, py1, cx, cy, px2, py2, i);
				var cp:MovieClip = new CurvePoint();
				cp.x = pt.x;
				cp.y = pt.y;
				cp.alpha = 0.2;
				cp.scaleX = cp.scaleY = 0.6;
				addChild(cp);
				curvePoints.push(cp);
			}
		}

		public function drawLinePoints (px1:Number, py1:Number, px2:Number, py2:Number):void {
			// Draws points on a line
			var i:Number;
			for (i = 0; i < 1; i+= 1/5) {
				var pt = {x:px1*(1-i)+px2*i, y:py1*(1-i)+py2*i};
				var cp:MovieClip = new CurvePoint();
				cp.x = pt.x;
				cp.y = pt.y;
				cp.alpha = 0.2;
				cp.scaleX = cp.scaleY = 0.6;
				addChild(cp);
				curvePoints.push(cp);
			}
		}


		// http://ibiblio.org/e-notes/Splines/Bezier.htm
		public function pennerPointOnCurve(p1x, p1y, cx, cy, p2x, p2y, t):Object {
			// Returns the points on a bezier curve for a given time (t is 0-1);
			// This is based on Robert Penner's Math.pointOnCurve() function
			// More information: http://actionscript-toolbox.com/samplemx_pathguide.php
			return {x:p1x + t*(2*(1-t)*(cx-p1x) + t*(p2x - p1x)),
					y:p1y + t*(2*(1-t)*(cy-p1y) + t*(p2y - p1y))};
			// Quadratic Bezier spline
		}

		public function updateCone(p_index:Number):void {
			// Based on the position of an icon, update the Main view
			var mainCone:Ase = Main(parent).getCone(p_index); // cone on the main view
			mainCone[d1] = (coneList[p_index].x - (sizeW/2)) / CONE_SCALE;
			mainCone[d2] = (coneList[p_index].y - (sizeH/2)) / CONE_SCALE * -1;
			Main(parent).redrawViewports();
		}

		public function setPointAlpha(p:Number, p_alpha:Number):void {
			if (Boolean(controlPoints[p])) controlPoints[p].alpha = p_alpha;
		}

		public function updatePoints():void {
			// Based on the position of points here, update the Main view
			Main(parent).p1[d1] = (p1.x - (sizeW/2)) / CONE_SCALE;
			Main(parent).p1[d2] = (p1.y - (sizeH/2)) / CONE_SCALE * -1;
			Main(parent).p2[d1] = (p2.x - (sizeW/2)) / CONE_SCALE;
			Main(parent).p2[d2] = (p2.y - (sizeH/2)) / CONE_SCALE * -1;

			var i:uint;
			for (i = 0; i < controlPoints.length; i++) {
				var mainPoint:Object = Main(parent).getControlPoint(i);
				var localPoint:Sprite = controlPoints[i];
				mainPoint[d1] = (localPoint.x - (sizeW/2)) / CONE_SCALE;
				mainPoint[d2] = (localPoint.y - (sizeH/2)) / CONE_SCALE * -1;
			}

			Main(parent).redrawViewports();
		}

		public function stopTravel():void {
			Tweener.removeTweens(ball);
			ball.x = p1.x;
			ball.y = p1.y;
		}
		public function startTravel(bezierList:Array, animTime:Number, animDelay:Number):void {
			// Plays the ball using the bezier values
			// Creates list of control points
			var i:uint;
			var myBezierList:Array = [];
			var offsetX:Number = sizeW/2;
			var offsetY:Number = sizeH/2;
			for (i = 0; i < bezierList.length; i++) {
				myBezierList.push({x:offsetX + bezierList[i][d1] * CONE_SCALE, y:offsetY + bezierList[i][d2] * -CONE_SCALE});
			}
			Tweener.addTween(ball, {x:p2.x, y:p2.y, _bezier:myBezierList, time:animTime, delay:animDelay, transition:"linear"});
		
		}
	}
}
